<?php
/**
 *  @author    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');

class FOrder
{
    private $baseQuery;

    public function __construct($common, $date_from)
    {
        if (empty($date_from)) {
            $date_from = '1970-01-01';
        }
        $date_from = (new DateTime($date_from))->format('Y-m-d');
        $date_from = pSQL($date_from);
        $this->common = $common;
        $isIncremental = Configuration::get('SAMBA_INCREMENTAL') == 1;
        $date_field = $isIncremental ? 'date_upd' : 'date_add';
        $date_from = $isIncremental ? date('Y-m-d', strtotime('-7 days')) : (new DateTime($date_from))->format('Y-m-d');
        $date_from = pSQL($date_from);

        $this->baseQuery = "
            SELECT 
                o.current_state, 
                o.date_add, 
                o.date_upd, 
                o.delivery_date, 
                o.date_upd, 
                o.id_customer, 
                o.id_order, 
                a.postcode, 
                a.phone_mobile, 
                od.product_id, 
                od.product_attribute_id, 
                od.total_price_tax_incl, 
                od.product_quantity,
                cc.call_prefix,
                cu.email
            FROM 
                " . _DB_PREFIX_ . "orders AS o
            INNER JOIN 
                " . _DB_PREFIX_ . "address AS a ON o.id_address_delivery = a.id_address
            INNER JOIN 
                " . _DB_PREFIX_ . "order_detail AS od ON o.id_order = od.id_order
            INNER JOIN 
                " . _DB_PREFIX_ . "customer AS cu ON o.id_customer = cu.id_customer
            LEFT JOIN 
                " . _DB_PREFIX_ . "country AS cc ON cc.id_country = a.id_country
            WHERE 
                o.module!='x13allegro'
            AND 
                o.$date_field BETWEEN '$date_from' AND (NOW() + INTERVAL 2 HOUR)
            AND 
                cu.email NOT LIKE '%@allegro%'
        ";
    }

    public function genFeed($filePath = null)
    {
        $pageSize = 10000;
        $lastId = 0;
        $batchCount = 0;

        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('ORDERS');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('ORDERS');
        }

        $page = 0;
        while (true) {
            $orders = $this->fetchOrders($lastId, $pageSize);
            if (empty($orders)) {
                break;
            }

            $this->genFeedPage($orders, $xmlWriter);

            // Get the last order ID for pagination
            $lastId = end($orders)['id_order'];

            // Flush the data to the file every batch
            if ($isCron && ++$batchCount % 10 == 0) {
                file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
            }

            // Break if fewer orders than page size, indicating last batch
            if (count($orders) < $pageSize) {
                break;
            }

            $page++;
        }

        // Final flush to make sure we haven't missed anything
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        }

        // End the ORDERS element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }
    }

    private function fetchOrders($lastId, $limit)
    {
        $query = $this->baseQuery . " AND o.id_order > $lastId ORDER BY o.id_order LIMIT $limit";
        error_log("Executing query: $query\n", 3, "debug.log");
        $orders = Db::getInstance()->executeS($query);
        error_log("Fetched " . count($orders) . " orders\n", 3, "debug.log");
        return $orders;
    }

    private function genFeedPage($orders, $xmlWriter)
    {
        $common = $this->common;
        $variant_ids = $common->config['VARIANT_IDS'];
        $orders = $this->group_order_details($orders);

        foreach ($orders as $order_id => $ods) {
            $o = $ods[0];
            $order_id = $o['id_order'];
            $customer_id = (int)$o['id_customer'];

            $xmlWriter->startElement('ORDER');
            $status = $this->getStatus(($o['current_state']));
            $xmlWriter->writeElement('ORDER_ID', $order_id);
            $xmlWriter->writeElement('CUSTOMER_ID', $customer_id);
            $xmlWriter->writeElement('CREATED_ON', $common->dtIso($o['date_add']));
            $xmlWriter->writeElement('EMAIL', $o['email']);
            if ($status == 'finished') {
                $xmlWriter->writeElement('FINISHED_ON', $common->dtIso($o['date_upd']));
            }
            $xmlWriter->writeElement('STATUS', $status);
            if (!empty($o['postcode'])) {
                $xmlWriter->writeElement('ZIP_CODE', $o['postcode']);
            }
            if (!empty($o['phone_mobile'])) {
                $xmlWriter->writeElement('PHONE', $this->validatePhoneFormat($o['phone_mobile'], $o['call_prefix']));
            }
            $xmlWriter->startElement('ITEMS');
            foreach ($ods as $od) {
                $xmlWriter->startElement('ITEM');
                if ($variant_ids) {
                    $xmlWriter->writeElement('PRODUCT_ID', $od['product_id'] . '-' . $od['product_attribute_id']);
                } else {
                    $xmlWriter->writeElement('PRODUCT_ID', $od['product_id'] . '-0');
                }
                $xmlWriter->writeElement('PRICE', $od['total_price_tax_incl']);
                $xmlWriter->writeElement('AMOUNT', $od['product_quantity']);
                $xmlWriter->endElement();
            }
            $xmlWriter->endElement();
            $xmlWriter->endElement(); // ORDER
        }
        return count($orders);
    }

    private function getStatus($value)
    {
        $arr_create = explode(',', Configuration::get('SAMBA_ORDER_CREATE'));
        $arr_finished = explode(',', Configuration::get('SAMBA_ORDER_FINISHED'));
        $arr_canceled = explode(',', Configuration::get('SAMBA_ORDER_CANCELED'));
        if (in_array($value, $arr_create))
            return 'created';
        else if (in_array($value, $arr_finished))
            return 'finished';
        else if (in_array($value, $arr_canceled))
            return 'canceled';
        else
            return 'created';
    }

    public function validatePhoneFormat($phone, $CallPrefix)
    {
        // This will delete all unwanted chars from number (expect +)
        $phone_number = filter_var($phone, FILTER_SANITIZE_NUMBER_INT);
        // Check if phone number have prefix +xxx and 11 or 12 digits
        if (preg_match('/^\+[0-9]{1,2}[0-9]{3}[0-9]{7}$/', $phone_number)) {
            // Check if phone without prefix have 9 digit
        } else if (preg_match('/^[0-9]{9}$/', $phone_number)) {
            $phone_number = '+' . $CallPrefix . $phone_number;
        } else {
            $phone_number = '';
        }
        return $phone_number;
    }

    private function group_order_details($orders)
    {
        $os = array();
        $oid = null;
        $acc = array();
        foreach ($orders as $order) {
            $order_id = $order['id_order'];
            if ($oid != $order_id) {
                if (isset($oid)) {
                    $os[$oid] = $acc;
                }
                $acc = array();
                $oid = $order_id;
            }
            $acc[] = $order;
        }
        return $os;
    }
}
